/* Copyright (c) 1996 University of Utah.  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for NON-COMMERCIAL purposes and without fee is
 * hereby granted provided that this copyright notice appears in all
 * copies.
 *
 * See http://www.cs.utah.edu/~hamlet for more educational resources.
 *
 * The University of Utah MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The University of Utah SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package Root;

public class Func {
  String name;
  double posGuess;  /* First positive guess. Used in bisection */
  double negGuess;  /* First negative guess.  Used in bisection */
  double guess;     /* First guess.  Used in newtons */
  double plotStart; /* Where should the plot start */
  double plotStop;  /* and where should it stop */

  double xtick1;    /* Hardcoded ticks for the plot */
  double xtick2;
  double ytick1;
  double ytick2;

  Func(String nameIn, double posGuessIn, double negGuessIn,
                      double guessIn,
                      double plotStartIn, double plotStopIn) {
    name = nameIn;
    posGuess = posGuessIn;
    negGuess = negGuessIn;
    guess = guessIn;
    plotStart = plotStartIn;
    plotStop = plotStopIn;
  }


}
  
class FuncArray extends Object {
  Func array[] = new Func[10];
  int size;
  Object parent;

  static final int ROOT = 0;

  static final int X_SQUARED  = 0;
  static final int X_CUBED    = 1;
  static final int MIKE_FUN   = 2;
  static final int COS_10_X   = 3;  /* This one takes 14 guesses in newton */
  static final int X          = 4;  
  static final int COS_X      = 5;  
  static final int SIN_X      = 6;  
  static final int X_SQUARED_SMALL = 7;
  static final int KEN_FUN    = 8;
  

  FuncArray(Object parentIn) {
  
    parent = parentIn;

    /* posGuess, negGuess, guess, plotStart, plotStop */
    array[X_SQUARED] = new Func("x^2 - 5",            -5,    1,  -5,   -7,   3);
    array[X_CUBED]   = new Func("x^3 - 5",             5,   -5,   5,   -7,   8);
    array[MIKE_FUN]  = new Func("sin(5x) + x^2 - 3x", -2,    1,   3,   -5,   5);
    array[COS_10_X]  = new Func("cos(10x) + 4x - 3",   3.5, -.2,  3.3, -1.2, 4);
    array[X]         = new Func("x",                   5,   -5,   4,   -7,   7); 
    array[COS_X]     = new Func("cos(x)",             -1,    2,  -3,   -5,   5); 
    array[SIN_X]     = new Func("sin(x)",             -4,    4,   1.5, -5,   5);
    array[X_SQUARED_SMALL] = new Func("x^2 + 1",     -8,   8, -8, -9, 9);
    array[KEN_FUN]   = new Func("ln(x^2 - (4/5)x + 1)", -2.5,    .2, 1.5, -3,   3); 

    size = 9;
  }

  public double doFunc(double x) {
    int whichFunc = 0;
    double ret;

    whichFunc = ((Root) parent).curFunc;
    
    switch (whichFunc) {
    
    case X_SQUARED:
      ret =  (x*x - 5);
      break;
    case X_CUBED:
      ret =  (x*x*x - 5);
      break;
    case MIKE_FUN:
      ret =  (Math.sin(5 * x) + (x * x) - (3 * x));
      break;
    case COS_10_X:
      ret =  (Math.cos(10 * x) + (4 * x) - 3 );
      break;
    case X:
      ret =  x;
      break;
    case COS_X:
      ret =  Math.cos(x);
      break;
    case SIN_X:
      ret =  Math.sin(x);
      break;
    case X_SQUARED_SMALL:
      ret =  (x*x + 1);
      break;
    case KEN_FUN:
      ret =  ( Math.log(x*x - 4.0/5.0*x +1.0)); 
      break;
    default:
      System.out.println("Unknown function " + whichFunc);
      ret = 0;
    }
	      
    return (ret);
  }

  /* This is used in Newton's method */
  public double doDiriv(double x) {
    int whichFunc = 0;
    double ret;

    whichFunc = ((Root) parent).curFunc;
    
    switch (whichFunc) {
    
    case X_SQUARED:
      ret =  (2*x);
      break;
    case X_CUBED:
      ret =  (3*(x*x));
      break;
    case MIKE_FUN:
      ret =  (5*Math.cos(5*x) + (2*x) - 3);
      break;
    case COS_10_X:
      ret =  (-10*Math.sin(10*x) +4);
      break;
    case X:
      ret =  1;
      break;
    case COS_X:
      ret =  - Math.sin(x);
      break;
    case SIN_X:
      ret =  Math.cos(x);
      break;
    case X_SQUARED_SMALL:
      ret =  (2*x);
      break;
    case KEN_FUN:
      ret =  ( (2*x - 4./5.) / (x*x - 4./5.*x + 1) );
      break;
    default:
      System.out.println("Unknown function " + whichFunc);
      ret = 0;
    }
	      
    return (ret);
  }

    
    

}    
